package ca.mcgill.sable.abc.impact.visualizer.model;

import java.util.List;

import org.eclipse.core.runtime.Platform;

import soot.Modifier;
import soot.SootField;
import soot.SootMethod;
import soot.Type;
import abc.impact.impact.FieldName;
import abc.impact.impact.MethodSignature;
import abc.impact.utils.ImpactUtil;
import abc.weaving.aspectinfo.DeclareParentsExt;
import abc.weaving.aspectinfo.FieldSig;
import abc.weaving.aspectinfo.InAspect;
import abc.weaving.aspectinfo.IntertypeConstructorDecl;
import abc.weaving.aspectinfo.IntertypeFieldDecl;
import abc.weaving.aspectinfo.IntertypeMethodDecl;
import abc.weaving.aspectinfo.MethodSig;
import ca.mcgill.sable.abc.impact.visualizer.util.ImpactPluginUtil;

public final class InAspectTreeNode extends AbstractTreeNode 
	implements IActPosition{

	private final InAspect inAspect;
	
	public InAspectTreeNode(ITreeNode parent, final InAspect inAspect) {
		super(parent);
		this.inAspect = inAspect;
		
		if (inAspect instanceof IntertypeMethodDecl) {
			IntertypeMethodDecl methodDecl = (IntertypeMethodDecl)inAspect;
			MethodSig mSig = methodDecl.getTarget();
			SootMethod sm = ImpactUtil.getITDMethodTarget(mSig);

			MethodSignature mSign = new MethodSignature(sm);
			StringBuffer nameb = new StringBuffer();
			nameb.append(sm.getDeclaringClass().getShortName());
			nameb.append('.');
			nameb.append(mSign.getMethodName());
			nameb.append('(');
			List<Type> types = mSign.getParams(); 
			for (Type type : types) {
				nameb.append(type);
				nameb.append(",");
			}
			if (! types.isEmpty()) nameb.setLength(nameb.length() - 1);
			nameb.append(')');
			this.name = nameb.toString();
			
			int realMod = ImpactUtil.getRealModifiers(methodDecl);
			if (Modifier.isPrivate(realMod)) {
				this.image = ImageFactory.ITDMETHOD_PRI.getImage();
			} else if (Modifier.isPublic(realMod)) {
				this.image = ImageFactory.ITDMETHOD_PUB.getImage();
			} else { // would be default, since no protect
				this.image = ImageFactory.ITDMETHOD_DEF.getImage();
			}
		} else if (inAspect instanceof IntertypeFieldDecl) {
			IntertypeFieldDecl fieldDecl = (IntertypeFieldDecl)inAspect;
			FieldSig itdFieldSig = fieldDecl.getTarget();
			SootField sf = ImpactUtil.getITDFieldTarget(itdFieldSig);
			
			FieldName fieldName = new FieldName(sf);
			StringBuffer nameb = new StringBuffer();
			nameb.append(sf.getDeclaringClass().getShortName());
			nameb.append('.');
			nameb.append(fieldName.getFieldName());
			this.name = nameb.toString();
			
			int realMod = ImpactUtil.getRealModifiers(itdFieldSig);
			if (Modifier.isPrivate(realMod)) {
				this.image = ImageFactory.ITDFIELD_PRI.getImage();
			} else if (Modifier.isPublic(realMod)) {
				this.image = ImageFactory.ITDFIELD_PUB.getImage();
			} else { // would be default, since no protect
				this.image = ImageFactory.ITDFIELD_DEF.getImage();
			}
		} else if (inAspect instanceof IntertypeConstructorDecl) {
			IntertypeConstructorDecl conDecl = (IntertypeConstructorDecl)inAspect;
			SootMethod sm = ImpactUtil.getITDConstructorTarget(conDecl);
			
			MethodSignature mSign = new MethodSignature(sm);
			StringBuffer nameb = new StringBuffer();
			nameb.append(sm.getDeclaringClass().getShortName());
			nameb.append('(');
			List<Type> types = mSign.getParams(); 
			for (Type type : types) {
				nameb.append(type);
				nameb.append(",");
			}
			if (! types.isEmpty()) nameb.setLength(nameb.length() - 1);
			nameb.append(')');
			this.name = nameb.toString();

			int realMod = ImpactUtil.getRealModifiers(conDecl);
			if (Modifier.isPrivate(realMod)) {
				this.image = ImageFactory.ITDCONSTRUCTOR_DEF.getImage();
			} else if (Modifier.isPublic(realMod)) {
				this.image = ImageFactory.ITDCONSTRUCTOR_PUB.getImage();
			} else { // would be default, since no protect
				this.image = ImageFactory.ITDCONSTRUCTOR_DEF.getImage();
			}
		} else if (inAspect instanceof DeclareParentsExt) {
			DeclareParentsExt pe = (DeclareParentsExt)inAspect;
			this.name = "declare parents: " + pe.getPattern().getPattern()
					+ " extends "
					+ pe.getParent().getSootClass().getShortName();
			
			this.image = ImageFactory.DEC_PARENTS.getImage();
		} else { //should not happen
			this.name = this.getClass() + ": unknow InAspect " + inAspect.getClass();
		}
	}
	
	public InAspect getInAspect() {
		return inAspect;
	}

	public Object getAdapter(Class adapter) {
		return Platform.getAdapterManager().getAdapter(this, adapter);
	}

	public RevealData getRevealData() {
		return new RevealData(ImpactPluginUtil.getFileFrom(inAspect.getAspect()),
				inAspect.getPosition());
	}
}
