package ca.mcgill.sable.abc.impact.visualizer.model;

import java.lang.reflect.Modifier;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.ui.JavaUI;

import soot.SootMethod;
import soot.Type;
import abc.impact.impact.LookupChange;
import abc.impact.impact.LookupImpact;
import abc.impact.impact.MethodSignature;
import abc.impact.utils.ImpactUtil;
import ca.mcgill.sable.abc.impact.visualizer.actions.RunImpactAnalysisActionDelegate;
import ca.mcgill.sable.abc.impact.visualizer.util.ImpactPluginUtil;

// NOTE this class is used in plug-in decrators extension
public final class LookupImpactCurrentTypeTreeNode extends AbstractTreeNode
	implements IActJavaElement, IProvideJavaElementImage {

	final private LookupImpact lookupImpact;
	final private LookupChange lookupChange;
	private final IJavaElement jEle;
	
	public LookupImpactCurrentTypeTreeNode(ITreeNode parent, final LookupImpact lookupImpact, final LookupChange lookupChange) {
		super(parent);
		this.lookupImpact = lookupImpact;
		this.lookupChange = lookupChange;
		
		SootMethod currentMethod = lookupChange.getCurrentMethod();
		MethodSignature mSig = new MethodSignature(currentMethod);
		List<Type> params = mSig.getParams();
		
		StringBuffer nameb = new StringBuffer();
		nameb.append(currentMethod.getDeclaringClass().getName());
		if (! mSig.getMethodName().equals(SootMethod.constructorName)) {
			nameb.append(".");
			nameb.append(mSig.getMethodName());
		}
		nameb.append('(');
		for (Type type : params) {
			nameb.append(type);
			nameb.append(',');
		}
		if (!params.isEmpty()) nameb.setLength(nameb.length()-1);
		nameb.append(')');
		this.name = nameb.toString();
		
		int mod = ImpactUtil.getRealModifiers(currentMethod);
		if (Modifier.isPublic(mod)) {
			this.image = JavaUI.getSharedImages().getImage(
					org.eclipse.jdt.ui.ISharedImages.IMG_OBJS_PUBLIC);
		} else if (Modifier.isProtected(mod)) {
			this.image = JavaUI.getSharedImages().getImage(
					org.eclipse.jdt.ui.ISharedImages.IMG_OBJS_PROTECTED);
		} else if (Modifier.isPrivate(mod)) {
			this.image = JavaUI.getSharedImages().getImage(
					org.eclipse.jdt.ui.ISharedImages.IMG_OBJS_PRIVATE);
		} else {
			this.image = JavaUI.getSharedImages().getImage(
					org.eclipse.jdt.ui.ISharedImages.IMG_OBJS_DEFAULT);
		}
		
		IJavaElement ret = null;
		IType jType = (IType)ImpactPluginUtil.findJavaType(
				RunImpactAnalysisActionDelegate.getCurrentProject(), 
				currentMethod.getDeclaringClass().getName());
		if (jType != null) {
			ret = jType;
			String [] jParams = new String[params.size()];
			for (int i = 0; i < params.size(); i++) {
				jParams[i] = ImpactPluginUtil.toJDTType(params.get(i));
			}
			IMethod jMethod = jType.getMethod(mSig.getMethodName(), jParams);
			if (jMethod.exists()) {
				ret = jMethod;
			}
		}
		this.jEle = ret;
	}
	
	public LookupImpact getLookupImpact() {
		return lookupImpact;
	}

	public LookupChange getLookupChange() {
		return lookupChange;
	}

	public IJavaElement getJavaElement() {
		return jEle;
	}

	public Object getAdapter(Class adapter) {
		return Platform.getAdapterManager().getAdapter(this, adapter);
	}
}